@@ -1,6 +1,6 @@
 /*
     DABlin - capital DAB experience
-    Copyright (C) 2015-2020 Stefan Pöschel
+    Copyright (C) 2015-2021 Stefan Pöschel
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@ static void usage(const char* exe) {
 					"  -p           Output PCM to stdout instead of using SDL\n"
 					"  -u           Output untouched audio stream to stdout instead of using SDL\n"
 					"  -I           Don't catch up on stream after interruption\n"
+					"  -Y           Initially disable Dynamic Label Plus (DL+)\n"
 					"  -S           Initially disable slideshow\n"
 					"  -L           Enable loose behaviour (e.g. PAD conformance)\n"
 					"  -F           Disable dynamic FIC messages (dynamic PTY, announcements)\n"
@@ -82,7 +83,7 @@ int main(int argc, char **argv) {
 
 	// option args
 	int c;
-	while((c = getopt(argc, argv, "hf:d:D:C:c:l:g:Gr:P:s:x:1puISLF")) != -1) {
+	while((c = getopt(argc, argv, "hf:d:D:C:c:l:g:Gr:P:s:x:1puIYSLF")) != -1) {
 		switch(c) {
 		case 'h':
 			usage(argv[0]);
@@ -140,6 +141,9 @@ int main(int argc, char **argv) {
 		case 'I':
 			options.disable_int_catch_up = true;
 			break;
+		case 'Y':
+			options.initially_disable_dl_plus = true;
+			break;
 		case 'S':
 			options.initially_disable_slideshow = true;
 			break;
@@ -251,6 +255,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 
 	dt_lto = FIC_ENSEMBLE::lto_none;
 
+	dl_plus_window.set_transient_for(*this);
 	slideshow_window.set_transient_for(*this);
 
 	ensemble_update_progress.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::EnsembleUpdateProgressEmitted));
@@ -260,6 +265,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	pad_change_dynamic_label.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeDynamicLabelEmitted));
 	pad_change_slide.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeSlideEmitted));
 	do_rec_status_update.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoRecStatusUpdateEmitted));
+	do_datetime_sync.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoDateTimeSyncEmitted));
 	do_datetime_update.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoDateTimeUpdateEmitted));
 
 	if(options.source_format == EnsembleSource::FORMAT_ETI)
@@ -406,6 +412,11 @@ void DABlinGTK::InitWidgets() {
 	tglbtn_slideshow.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_slideshow));
 	tglbtn_slideshow.set_tooltip_text("Slideshow (if available)");
 
+	tglbtn_dl_plus.set_label("DL+");
+	tglbtn_dl_plus.set_active(!options.initially_disable_dl_plus);
+	tglbtn_dl_plus.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_dl_plus));
+	tglbtn_dl_plus.set_tooltip_text("Dynamic Label Plus (if available)");
+
 	tglbtn_mute.set_image_from_icon_name("audio-volume-muted");
 	tglbtn_mute.signal_clicked().connect(sigc::mem_fun(*this, &DABlinGTK::on_tglbtn_mute));
 	tglbtn_mute.set_tooltip_text("Mute");
@@ -452,14 +463,15 @@ void DABlinGTK::InitWidgets() {
 	top_grid.attach_next_to(frame_combo_services, frame_label_ensemble, Gtk::POS_RIGHT, 1, 2);
 	top_grid.attach_next_to(frame_label_format, frame_combo_services, Gtk::POS_RIGHT, 1, 2);
 	top_grid.attach_next_to(tglbtn_record, frame_label_format, Gtk::POS_RIGHT, 1, 1);
-	top_grid.attach_next_to(label_record, tglbtn_record, Gtk::POS_RIGHT, 2, 1);
+	top_grid.attach_next_to(label_record, tglbtn_record, Gtk::POS_RIGHT, 3, 1);
 	top_grid.attach_next_to(tglbtn_slideshow, tglbtn_record, Gtk::POS_BOTTOM, 1, 1);
-	top_grid.attach_next_to(tglbtn_mute, tglbtn_slideshow, Gtk::POS_RIGHT, 1, 1);
+	top_grid.attach_next_to(tglbtn_dl_plus, tglbtn_slideshow, Gtk::POS_RIGHT, 1, 1);
+	top_grid.attach_next_to(tglbtn_mute, tglbtn_dl_plus, Gtk::POS_RIGHT, 1, 1);
 	top_grid.attach_next_to(vlmbtn, tglbtn_mute, Gtk::POS_RIGHT, 1, 1);
-	top_grid.attach_next_to(frame_label_dl, frame_combo_channels, Gtk::POS_BOTTOM, 7, 1);
+	top_grid.attach_next_to(frame_label_dl, frame_combo_channels, Gtk::POS_BOTTOM, 8, 1);
 	top_grid.attach_next_to(frame_label_datetime, frame_label_dl, Gtk::POS_BOTTOM, 2, 1);
-	top_grid.attach_next_to(frame_label_asu, frame_label_datetime, Gtk::POS_RIGHT, 5, 1);
-	top_grid.attach_next_to(progress_position, frame_label_datetime, Gtk::POS_BOTTOM, 7, 1);
+	top_grid.attach_next_to(frame_label_asu, frame_label_datetime, Gtk::POS_RIGHT, 6, 1);
+	top_grid.attach_next_to(progress_position, frame_label_datetime, Gtk::POS_BOTTOM, 8, 1);
 
 	show_all_children();
 	progress_position.hide();	// invisible until progress updated
@@ -574,6 +586,9 @@ void DABlinGTK::SetService(const LISTED_SERVICE& service) {
 		label_dl.set_label("");
 		frame_label_dl.set_tooltip_text("");
 
+		dl_plus_window.hide();
+		dl_plus_window.ClearDLPlusInfo();
+
 		if(!service.HasSLS()) {
 			slideshow_window.hide();
 			slideshow_window.ClearSlide();
@@ -617,6 +632,7 @@ void DABlinGTK::UpdateAnnouncementSupport(const LISTED_SERVICE& service) {
 
 	// assemble text - iterate through all supported announcement types
 	std::string ac_str;
+	size_t count = 0;
 	for(int type = 0; type < 16; type++) {
 		uint16_t type_flag = 1 << type;
 		if(!(service.asu_flags & type_flag))
@@ -637,8 +653,13 @@ void DABlinGTK::UpdateAnnouncementSupport(const LISTED_SERVICE& service) {
 			}
 		}
 
-		if(!ac_str.empty())
-			ac_str += " + ";
+		// regularly insert line break
+		if(!ac_str.empty()) {
+			ac_str += (count % 6 == 0) ? "\n" : " ";
+			ac_str += "+ ";
+		}
+		count++;
+
 		if(!asw_color.empty())
 			ac_str += "<span bgcolor='" + asw_color + "'>";
 		ac_str += FICDecoder::ConvertASuTypeToString(type);
@@ -847,6 +868,13 @@ void DABlinGTK::on_vlmbtn(double value) {
 		tglbtn_mute.clicked();
 }
 
+void DABlinGTK::on_tglbtn_dl_plus() {
+	if(tglbtn_dl_plus.get_active())
+		dl_plus_window.TryToShow();
+	else
+		dl_plus_window.hide();
+}
+
 void DABlinGTK::on_tglbtn_slideshow() {
 	if(tglbtn_slideshow.get_active())
 		slideshow_window.TryToShow();
@@ -1005,7 +1033,9 @@ void DABlinGTK::TrySwitch(Gtk::ComboBox& combo, Glib::RefPtr<Gtk::ListStore>& co
 }
 
 bool DABlinGTK::HandleConfigureEvent(GdkEventConfigure* /*configure_event*/) {
-	// move together with slideshow window
+	// move together with children windows
+	if(dl_plus_window.get_visible())
+		dl_plus_window.AlignToParent();
 	if(slideshow_window.get_visible())
 		slideshow_window.AlignToParent();
 	return false;
@@ -1113,6 +1143,10 @@ void DABlinGTK::FICChangeUTCDateTime(const FIC_DAB_DT& utc_dt) {
 	utc_dt_next = utc_dt;
 	dt_update = std::chrono::steady_clock::now();
 //	fprintf(stderr, "### time synced\n");
+
+	if(dt_lto != FIC_ENSEMBLE::lto_none)
+		DoDateTimeSync(FICDecoder::ConvertDateTimeToString(utc_dt, dt_lto, true));
+
 	ShowDateTime(true);
 }
 
@@ -1156,11 +1190,17 @@ void DABlinGTK::EnsembleDoRegularWork() {
 }
 
 void DABlinGTK::DoDateTimeUpdateEmitted() {
-	// display received date/time
+	// display updated date/time
 	frame_label_datetime.set_sensitive(true);
 	label_datetime.set_label(do_datetime_update.Pop());
 }
 
+void DABlinGTK::DoDateTimeSyncEmitted() {
+	// display synced date/time
+	frame_label_datetime.set_sensitive(true);
+	frame_label_datetime.set_tooltip_text("Last synced date/time: " + do_datetime_sync.Pop());
+}
+
 void DABlinGTK::FICDiscardedFIB() {
 	fprintf(stderr, "\x1B[33m" "(FIB)" "\x1B[0m" " ");
 }
@@ -1183,6 +1223,7 @@ void DABlinGTK::on_combo_channels() {
 	frame_label_ensemble.set_tooltip_text("");
 
 	frame_label_datetime.set_sensitive(false);
+	frame_label_datetime.set_tooltip_text("");
 	label_datetime.set_label("");
 
 	utc_dt_curr = FIC_DAB_DT();
@@ -1280,6 +1321,13 @@ void DABlinGTK::PADChangeDynamicLabelEmitted() {
 		label_dl.set_label("");
 		frame_label_dl.set_tooltip_text("");
 	}
+
+	// if present, forward DL Plus updates
+	if(!dl.dl_plus_objects.empty()) {
+		dl_plus_window.UpdateDLPlusInfo(dl);
+		if(tglbtn_dl_plus.get_active())
+			dl_plus_window.TryToShow();
+	}
 }
 
 void DABlinGTK::PADChangeSlideEmitted() {
