@@ -1,6 +1,6 @@
 /*
     DABlin - capital DAB experience
-    Copyright (C) 2015-2020 Stefan Pöschel
+    Copyright (C) 2015-2021 Stefan Pöschel
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,9 @@
 #ifndef DABLIN_GTK_H_
 #define DABLIN_GTK_H_
 
+// eti_player.h, that indirectly includes SDL.h, must be included before <string> or it won't compile on OS X
+#include "eti_player.h"
+
 #include <algorithm>
 #include <list>
 #include <mutex>
@@ -30,9 +33,9 @@
 
 #include <gtkmm.h>
 
+#include "dablin_gtk_dl_plus.h"
 #include "dablin_gtk_sls.h"
 #include "eti_source.h"
-#include "eti_player.h"
 #include "edi_source.h"
 #include "edi_player.h"
 #include "fic_decoder.h"
@@ -88,6 +91,7 @@ struct DABlinGTKOptions {
 	bool untouched_output;
 	bool disable_int_catch_up;
 	int gain;
+	bool initially_disable_dl_plus;
 	bool initially_disable_slideshow;
 	bool loose;
 	bool disable_dyn_fic_msgs;
@@ -104,6 +108,7 @@ DABlinGTKOptions() :
 	untouched_output(false),
 	disable_int_catch_up(false),
 	gain(DAB_LIVE_SOURCE_CHANNEL::auto_gain),
+	initially_disable_dl_plus(false),
 	initially_disable_slideshow(false),
 	loose(false),
 	disable_dyn_fic_msgs(false)
@@ -172,6 +177,7 @@ private:
 	int switch_service_scids;
 	bool switch_service_applied;
 
+	DABlinGTKDLPlusWindow dl_plus_window;
 	DABlinGTKSlideshowWindow slideshow_window;
 
 	EnsembleSource *ensemble_source;
@@ -247,6 +253,7 @@ private:
 	Gtk::Label label_record;
 
 	Gtk::ToggleButton tglbtn_slideshow;
+	Gtk::ToggleButton tglbtn_dl_plus;
 	Gtk::ToggleButton tglbtn_mute;
 	Gtk::VolumeButton vlmbtn;
 
@@ -274,6 +281,7 @@ private:
 	void on_tglbtn_record();
 	void on_tglbtn_mute();
 	void on_vlmbtn(double value);
+	void on_tglbtn_dl_plus();
 	void on_tglbtn_slideshow();
 	void on_combo_channels();
 	void on_combo_services();
@@ -289,6 +297,10 @@ private:
 	void DoRecStatusUpdateEmitted();
 	void UpdateRecStatus(bool decoding);
 
+	GTKDispatcherQueue<std::string> do_datetime_sync;
+	void DoDateTimeSync(const std::string& dt_str) {do_datetime_sync.PushAndEmit(dt_str);}
+	void DoDateTimeSyncEmitted();
+
 	GTKDispatcherQueue<std::string> do_datetime_update;
 	void DoDateTimeUpdate(const std::string& dt_str) {do_datetime_update.PushAndEmit(dt_str);}
 	void DoDateTimeUpdateEmitted();
