@@ -20,8 +20,9 @@
 
 
 // --- SuperframeFilter -----------------------------------------------------------------
-SuperframeFilter::SuperframeFilter(SubchannelSinkObserver* observer, bool decode_audio) : SubchannelSink(observer, "aac") {
+SuperframeFilter::SuperframeFilter(SubchannelSinkObserver* observer, bool decode_audio, bool enable_float32) : SubchannelSink(observer, "aac") {
 	this->decode_audio = decode_audio;
+	this->enable_float32 = enable_float32;
 
 	aac_dec = nullptr;
 
@@ -84,9 +85,17 @@ void SuperframeFilter::Feed(const uint8_t *data, size_t len) {
 		return;
 
 
+	int total_corr_count;
+	bool uncorr_errors;
+
 	// append RS coding on copy
 	memcpy(sf, sf_raw, sf_len);
-	rs_dec.DecodeSuperframe(sf, sf_len);
+	rs_dec.DecodeSuperframe(sf, sf_len, total_corr_count, uncorr_errors);
+
+	// forward statistics if errors present
+	if(total_corr_count || uncorr_errors)
+		observer->FECInfo(total_corr_count, uncorr_errors);
+
 
 	if(!CheckSync()) {
 		if(sync_frames == 0)
@@ -98,7 +107,6 @@ void SuperframeFilter::Feed(const uint8_t *data, size_t len) {
 	if(sync_frames) {
 		fprintf(stderr, "SuperframeFilter: Superframe sync succeeded after %d frame(s)\n", sync_frames);
 		sync_frames = 0;
-		ResetPAD();
 	}
 
 
@@ -118,8 +126,7 @@ void SuperframeFilter::Feed(const uint8_t *data, size_t len) {
 		uint16_t au_crc_stored = au_data[au_len-2] << 8 | au_data[au_len-1];
 		uint16_t au_crc_calced = CalcCRC::CalcCRC_CRC16_CCITT.Calc(au_data, au_len - 2);
 		if(au_crc_stored != au_crc_calced) {
-			fprintf(stderr, "\x1B[31m" "(AU #%d)" "\x1B[0m" " ", i);
-			ResetPAD();
+			observer->AudioError("AU #" + std::to_string(i));
 			continue;
 		}
 
@@ -153,14 +160,11 @@ void SuperframeFilter::CheckForPAD(const uint8_t *data, size_t len) {
 		}
 	}
 
-	if(!present)
-		ResetPAD();
-}
-
-void SuperframeFilter::ResetPAD() {
-	// required to reset internal state of PAD parser (in case of omitted CI list)
-	uint8_t zero_fpad[FPAD_LEN] = {0x00};
-	observer->ProcessPAD(nullptr, 0, true, zero_fpad);
+	// assume zero bytes F-PAD, if no DSE present
+	if(!present) {
+		uint8_t zero_fpad[FPAD_LEN] = {0x00};
+		observer->ProcessPAD(nullptr, 0, true, zero_fpad);
+	}
 }
 
 
@@ -213,12 +217,12 @@ bool SuperframeFilter::CheckSync() {
 
 void SuperframeFilter::ProcessFormat() {
 	// output format
-	const char *stereo_mode = (sf_format.aac_channel_mode || sf_format.ps_flag) ? "Stereo" : "Mono";
-	const char *surround_mode;
+	std::string core_mode = (sf_format.aac_channel_mode || sf_format.ps_flag) ? "Stereo" : "Mono";
+	std::string surround_mode;
 
 	switch(sf_format.mpeg_surround_config) {
 	case 0:
-		surround_mode = nullptr;
+		// no surround
 		break;
 	case 1:
 		surround_mode = "Surround 5.1";
@@ -231,22 +235,17 @@ void SuperframeFilter::ProcessFormat() {
 		break;
 	}
 
-	int bitrate = sf_len / 120 * 8;
-
-	std::stringstream ss;
-	ss << (sf_format.sbr_flag ? (sf_format.ps_flag ? "HE-AAC v2" : "HE-AAC") : "AAC-LC") << ", ";
-	ss << (sf_format.dac_rate ? 48 : 32) << " kHz ";
-	if(surround_mode)
-		ss << surround_mode << " (" << stereo_mode << " core) ";
-	else
-		ss << stereo_mode << " ";
-	ss << "@ " << bitrate << " kBit/s";
-	observer->FormatChange(ss.str());
+	AUDIO_SERVICE_FORMAT format;
+	format.codec = sf_format.sbr_flag ? (sf_format.ps_flag ? "HE-AAC v2" : "HE-AAC") : "AAC-LC";
+	format.samplerate_khz = sf_format.dac_rate ? 48 : 32;
+	format.mode = !surround_mode.empty() ? (surround_mode + " (" + core_mode + " core)") : core_mode;
+	format.bitrate_kbps = sf_len / 120 * 8;
+	observer->FormatChange(format);
 
 	if(decode_audio) {
 		delete aac_dec;
 #ifdef DABLIN_AAC_FAAD2
-		aac_dec = new AACDecoderFAAD2(observer, sf_format);
+		aac_dec = new AACDecoderFAAD2(observer, sf_format, enable_float32);
 #endif
 #ifdef DABLIN_AAC_FDKAAC
 		aac_dec = new AACDecoderFDKAAC(observer, sf_format);
@@ -309,7 +308,7 @@ void SuperframeFilter::ProcessUntouchedStream(const uint8_t *data, size_t len) {
 	au_bw.WriteAudioMuxLengthBytes();
 
 	const std::vector<uint8_t> latm_data = au_bw.GetData();
-	ForwardUntouchedStream(&latm_data[0], latm_data.size());
+	ForwardUntouchedStream(&latm_data[0], latm_data.size(), sf_format.GetAULengthMs());
 }
 
 
@@ -324,15 +323,15 @@ RSDecoder::~RSDecoder() {
 	free_rs_char(rs_handle);
 }
 
-void RSDecoder::DecodeSuperframe(uint8_t *sf, size_t sf_len) {
+void RSDecoder::DecodeSuperframe(uint8_t *sf, size_t sf_len, int& total_corr_count, bool& uncorr_errors) {
 //	// insert errors for test
 //	sf[0] ^= 0xFF;
 //	sf[10] ^= 0xFF;
 //	sf[20] ^= 0xFF;
 
 	int subch_index = sf_len / 120;
-	int total_corr_count = 0;
-	bool uncorr_errors = false;
+	total_corr_count = 0;
+	uncorr_errors = false;
 
 	// process all RS packets
 	for(int i = 0; i < subch_index; i++) {
@@ -357,10 +356,6 @@ void RSDecoder::DecodeSuperframe(uint8_t *sf, size_t sf_len) {
 			sf[pos * subch_index + i] = rs_packet[pos];
 		}
 	}
-
-	// output statistics if errors present (using ANSI coloring)
-	if(total_corr_count || uncorr_errors)
-		fprintf(stderr, "\x1B[36m" "(%d%s)" "\x1B[0m" " ", total_corr_count, uncorr_errors ? "+" : "");
 }
 
 
@@ -416,7 +411,9 @@ AACDecoder::AACDecoder(std::string decoder_name, SubchannelSinkObserver* observe
 
 #ifdef DABLIN_AAC_FAAD2
 // --- AACDecoderFAAD2 -----------------------------------------------------------------
-AACDecoderFAAD2::AACDecoderFAAD2(SubchannelSinkObserver* observer, SuperframeFormat sf_format) : AACDecoder("FAAD2", observer, sf_format) {
+AACDecoderFAAD2::AACDecoderFAAD2(SubchannelSinkObserver* observer, SuperframeFormat sf_format, bool float32) : AACDecoder("FAAD2", observer, sf_format) {
+	this->float32 = float32;
+
 	// ensure features
 	unsigned long cap = NeAACDecGetCapabilities();
 	if(!(cap & LC_DEC_CAP))
@@ -431,7 +428,7 @@ AACDecoderFAAD2::AACDecoderFAAD2(SubchannelSinkObserver* observer, SuperframeFor
 	if(!config)
 		throw std::runtime_error("AACDecoderFAAD2: error while NeAACDecGetCurrentConfiguration");
 
-	config->outputFormat = FAAD_FMT_FLOAT;
+	config->outputFormat = float32 ? FAAD_FMT_FLOAT : FAAD_FMT_16BIT;
 	config->dontUpSampleImplicitSBR = 0;
 
 	if(NeAACDecSetConfiguration(handle, config) != 1)
@@ -444,7 +441,7 @@ AACDecoderFAAD2::AACDecoderFAAD2(SubchannelSinkObserver* observer, SuperframeFor
 	if(init_result != 0)
 		throw std::runtime_error("AACDecoderFAAD2: error while NeAACDecInit2: " + std::string(NeAACDecGetErrorMessage(-init_result)));
 
-	observer->StartAudio(output_sr, output_ch, true);
+	observer->StartAudio(output_sr, output_ch, float32);
 }
 
 AACDecoderFAAD2::~AACDecoderFAAD2() {
@@ -455,7 +452,7 @@ void AACDecoderFAAD2::DecodeFrame(uint8_t *data, size_t len) {
 	// decode audio
 	uint8_t* output_frame = (uint8_t*) NeAACDecDecode(handle, &dec_frameinfo, data, len);
 	if(dec_frameinfo.error)
-		fprintf(stderr, "\x1B[35m" "(AAC)" "\x1B[0m" " ");
+		observer->AudioWarning("AAC");
 
 	// abort, if no output at all
 	if(dec_frameinfo.bytesconsumed == 0 && dec_frameinfo.samples == 0)
@@ -464,7 +461,7 @@ void AACDecoderFAAD2::DecodeFrame(uint8_t *data, size_t len) {
 	if(dec_frameinfo.bytesconsumed != len)
 		throw std::runtime_error("AACDecoderFAAD2: NeAACDecDecode did not consume all bytes");
 
-	observer->PutAudio(output_frame, dec_frameinfo.samples * 4);
+	observer->PutAudio(output_frame, dec_frameinfo.samples * (float32 ? 4 : 2));
 }
 #endif
 
@@ -535,7 +532,7 @@ void AACDecoderFDKAAC::DecodeFrame(uint8_t *data, size_t len) {
 	// decode audio
 	result = aacDecoder_DecodeFrame(handle, (short int*) output_frame, output_frame_len / 2, 0);
 	if(result != AAC_DEC_OK)
-		fprintf(stderr, "\x1B[35m" "(AAC)" "\x1B[0m" " ");
+		observer->AudioWarning("AAC");
 	if(!IS_OUTPUT_VALID(result))
 		return;
 
