@@ -150,7 +150,7 @@ bool MOTObject::ParseCheckHeader(MOT_FILE& target_file) {
 		case 0x0C:	// ContentName
 			if(data_len == 0)
 				return false;
-			file.content_name = FICDecoder::ConvertTextToUTF8(&data[offset + 1], data_len - 1, data[offset] >> 4, true, &file.content_name_charset);
+			file.content_name = CharsetTools::ConvertTextToUTF8(&data[offset + 1], data_len - 1, data[offset] >> 4, true, &file.content_name_charset);
 			new_content_name = file.content_name;
 //			fprintf(stderr, "ContentName: '%s'\n", file.content_name.c_str());
 			break;
